package routes_test

import (
	"context"
	"gitea.deepak.science/deepak/gogmagog/models"
	"gitea.deepak.science/deepak/gogmagog/routes"
	// "gitea.deepak.science/deepak/gogmagog/tokens"
	"github.com/stretchr/testify/assert"
	"net/http"
	"net/http/httptest"
	"strings"
	"testing"
)

func TestEmptyCurrentPlan(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	router := routes.NewCurrentPlanRouter(m)
	req, _ := http.NewRequestWithContext(sampleContext, "GET", "/", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusNotFound, status)
}

func TestEmptyCurrentPlanUnauth(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	router := routes.NewCurrentPlanRouter(m)
	req, _ := http.NewRequestWithContext(context.Background(), "GET", "/", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusUnauthorized, status)
}

func TestErrorCurrentPlan(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getErrorModel("Model always errors")

	router := routes.NewCurrentPlanRouter(m)
	req, _ := http.NewRequestWithContext(sampleContext, "GET", "/", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)
	// We pass in the date as a time.time so it makes sense that it comes back with a midnight timestamp.
	expected := `Internal Server Error`
	assert.Equal(expected, strings.TrimSpace(rr.Body.String()))
}

func TestEmptyCurrentPlanErrorWriter(t *testing.T) {
	// set up
	assert := assert.New(t)

	m := getEmptyModel()
	m.AddCurrentPlan(&models.CurrentPlan{PlanID: 3}, 3)

	router := routes.NewCurrentPlanRouter(m)
	req, _ := http.NewRequestWithContext(sampleContext, "GET", "/", nil)

	rr := NewBadWriter()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusInternalServerError, status)

}

func TestSingleCurrentPlan(t *testing.T) {
	// set up
	assert := assert.New(t)
	m := getEmptyModel()
	m.AddCurrentPlan(&models.CurrentPlan{PlanID: 3}, 3)

	router := routes.NewCurrentPlanRouter(m)
	req, _ := http.NewRequestWithContext(sampleContext, "GET", "/", nil)

	rr := httptest.NewRecorder()

	// function under test
	router.ServeHTTP(rr, req)

	// check results
	status := rr.Code
	assert.Equal(http.StatusOK, status)
	// We pass in the date as a time.time so it makes sense that it comes back with a midnight timestamp.
	expected := `{
		"plan_id": 3,
		"user_id": 3
	}`
	assert.JSONEq(expected, rr.Body.String())
	contentType := rr.Header().Get("Content-Type")
	assert.Equal("application/json", contentType)
}
